/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2016 OX Software GmbH, Germany. info@open-xchange.com
 *
 * @author Mario Scheliga <mario.scheliga@open-xchange.com>
 */

define('io.ox/settings/accounts/settings/pane', [
    'io.ox/core/extensions',
    'io.ox/core/tk/dialogs',
    'io.ox/keychain/api',
    'io.ox/keychain/model',
    'io.ox/core/folder/api',
    'io.ox/settings/util',
    'io.ox/core/notifications',
    'io.ox/backbone/mini-views',
    'io.ox/backbone/mini-views/listutils',
    'io.ox/backbone/disposable',
    'io.ox/core/api/filestorage',
    'settings!io.ox/core',
    'settings!io.ox/mail',
    'io.ox/core/api/account',
    'gettext!io.ox/settings/accounts',
    'io.ox/backbone/mini-views/settings-list-view',
    'withPluginsFor!keychainSettings'
], function (ext, dialogs, api, keychainModel, folderAPI, settingsUtil, notifications, mini, listUtils, DisposableView, filestorageApi, coreSettings, mailSettings, accounts, gt, ListView) {

    'use strict';

    // make sure changes get saved
    coreSettings.on('change:security/acceptUntrustedCertificates', function () {
        this.save();
    });

    var collection,

        createExtpointForSelectedAccount = function (args) {
            if (args.data.id !== undefined && args.data.accountType !== undefined) {
                ext.point('io.ox/settings/accounts/' + args.data.accountType + '/settings/detail').invoke('draw', args.data.node, args);
            }
        },

        onAddSubmodule = function (e) {
            e.preventDefault();
            var submodule = e.data.submodule;
            // looks like oauth?
            if ('reauthorize' in submodule) {
                var win = window.open(ox.base + '/busy.html', '_blank', 'height=600, width=800, resizable=yes, scrollbars=yes');
                submodule.createInteractively(win);
            } else {
                submodule.createInteractively(e);
            }
        },

        drawAddButton = function () {
            var submodules = _(api.submodules).filter(function (submodule) {
                return !submodule.canAdd || submodule.canAdd.apply(this);
            });

            if (submodules.length === 0) return;

            return $('<div class="btn-group col-md-4 col-xs-4">').append(
                $('<a class="btn btn-primary dropdown-toggle pull-right" role="button" data-toggle="dropdown" href="#" aria-haspopup="true">').append(
                    $.txt(gt('Add account')), $.txt(' '),
                    $('<span class="caret">')
                ),
                $('<ul class="dropdown-menu" role="menu">').append(
                   _(submodules).map(function (submodule) {
                       return $('<li role="presentation">').append(
                           $('<a href="#" role="menuitem">')
                           .attr('data-actionname', submodule.actionName || submodule.id || '')
                           .text(submodule.displayName)
                           .on('click', { submodule: submodule }, onAddSubmodule)
                       );
                   })
               )
            );
        },

        drawCertificateValidation = function () {
            return $('<div class="form-group">').append(
                $('<div class="checkbox">').append(
                    $('<label class="control-label">').text(gt('Allow connections with untrusted certificates')).prepend(
                        new mini.CheckboxView({ name: 'security/acceptUntrustedCertificates', model: coreSettings }).render().$el
                    )
                )
            );
        },

        drawRecoveryButtonHeadline = function () {
            return $('<h2 class="sr-only">').text(gt('Password recovery'));
        },

        drawRecoveryButton = function () {
            return $('<div class="hint">').append(
                $.txt(
                    gt('For security reasons, all credentials are encrypted with your primary account password. ' +
                        'If you change your primary password, your external accounts might stop working. In this case, ' +
                        'you can use your old password to recover all account passwords:')
                ),
                $.txt(' '),
                $('<a href="#" data-action="recover">').text(gt('Recover passwords')).attr({
                    role: 'button',
                    title: gt('Recover passwords'),
                    'aria-label': gt('Recover passwords')
                })
                .on('click', function (e) {
                    e.preventDefault();
                    ox.load(['io.ox/keychain/secretRecoveryDialog']).done(function (srd) {
                        srd.show();
                    });
                })
            );
        },

        drawPane = function () {
            return $('<div class="io-ox-accounts-settings">').append(
                $('<div>').addClass('row').append(
                    $('<h1 class="col-md-8 col-xs-8">').text(gt('Accounts')),
                    drawAddButton()
                ),
                $('<ul class="list-unstyled list-group widget-list">')
            );
        },

        drawIcon = (function () {
            var icons = {
                mail: 'fa-envelope',
                xing: 'fa-xing',
                twitter: 'fa-twitter',
                google: 'fa-google',
                yahoo: 'fa-yahoo',
                linkedin: 'fa-linkedin',
                dropbox: 'fa-dropbox',
                msliveconnect: 'fa-windows'
            };
            return function (type) {
                var icon = $('<i class="account-icon fa" aria-hidden="true">');
                if (type === 'boxcom') {
                    // there is no fitting icon for box in fontawesome
                    return icon
                        .removeClass('fa')
                        .css({
                            'background-image': 'url(apps/themes/default/box_logo36.png)',
                            'background-size': 'cover',
                            height: '14px',
                            width: '14px',
                            margin: '13px 0 13px 13px',
                            padding: 0
                        });
                }
                icon.addClass(icons[type] || 'fa-circle');
                return icon;
            };
        })(),

        /**
         * getAccountState
         * Used to display a possible error message
         * in a DSC environment. Returns a jQuery node
         * containing the error. Will not be used in
         * non-dsc setups (atm)
         */
        getAccountState = function (data) {
            if (data.model.get('type') !== 'mail' && mailSettings.get('dsc/enabled', false) === false) return $();
            var wrapper = $('<div class="account-error-wrapper">'),
                node = $('<div class="account-error">'),
                icon = $('<i class="account-error-icon fa fa-exclamation-triangle">');

            accounts.getStatus(data.model.get('id')).done(function (status) {
                if (status[data.model.get('id')].status !== 'ok') {
                    wrapper.append(icon, node).show();
                    node.text(status[data.model.get('id')].message);
                }
            });

            return wrapper.hide();
        },

        AccountSelectView = DisposableView.extend({

            tagName: 'li',

            className: 'settings-list-item',

            events: {
                'click [data-action="edit"]': 'onEdit',
                'click [data-action="delete"]': 'onDelete'
            },

            initialize: function () {
                this.model.on('change', this.render, this);
            },

            render: function () {
                var self = this,
                    title = self.model.get('displayName');
                self.$el.attr({
                    'data-id': self.model.get('id'),
                    'data-accounttype': self.model.get('accountType')
                });

                self.$el.append(
                    drawIcon(self.model.get('accountType')),
                    listUtils.makeTitle(title),
                    getAccountState(this), // show a possible account error
                    listUtils.makeControls().append(
                        listUtils.appendIconText(
                            listUtils.controlsEdit({ 'aria-label': gt('Edit %1$s', title) }),
                            gt('Edit'),
                            'edit'
                        ),
                        self.model.get('id') !== 0 ? listUtils.controlsDelete({ title: gt('Delete %1$s', title) }) : $('<div class="remove-placeholder">')
                    ),
                    // some Filestorage accounts may contain errors, if thats the case show them
                    // support for standard and oauth accounts
                    self.model.get('accountType') !== 'mail' ? listUtils.drawError(filestorageApi.getAccountsCache().get(self.model) || filestorageApi.getAccountForOauth(self.model)) : ''
                );

                return self;
            },

            onDelete: function (e) {

                e.preventDefault();

                var account = { id: this.model.get('id'), accountType: this.model.get('accountType') },
                    self = this;

                require(['io.ox/core/tk/dialogs'], function (dialogs) {
                    new dialogs.ModalDialog({ async: true })
                    .text(gt('Do you really want to delete this account?'))
                    .addPrimaryButton('delete', gt('Delete account'), 'delete')
                    .addButton('cancel', gt('Cancel'), 'cancel')
                    .on('delete', function () {
                        var popup = this;
                        settingsUtil.yellOnReject(
                            api.remove(account).then(
                                function success() {
                                    folderAPI.list('1', { cache: false });
                                    if (self.disposed) {
                                        popup.close();
                                        return;
                                    }

                                    self.model.collection.remove(self.model);
                                    popup.close();
                                },
                                function fail() {
                                    popup.close();
                                }
                            )
                            .always(function () {
                                // update folder tree
                                require(['io.ox/core/api/account', 'io.ox/core/folder/api'], function (accountAPI, folderAPI) {
                                    accountAPI.getUnifiedInbox().done(function (unifiedInbox) {
                                        if (!unifiedInbox) return;
                                        var prefix = unifiedInbox.split('/')[0];
                                        folderAPI.pool.unfetch(prefix);
                                        folderAPI.refresh();
                                    });
                                });
                            })
                        );
                    })
                    .show();
                });
            },

            onEdit: function (e) {
                e.preventDefault();
                e.data = {
                    id: this.model.get('id'),
                    accountType: this.model.get('accountType'),
                    node: this.el
                };
                createExtpointForSelectedAccount(e);
            }
        });
    /**
     * Extension point for account settings detail view
     *
     * This extension point provides a list to manage accounts of the keyring.
     *
     * As an extension to basic extension points, accounts can implement a canAdd
     * attribute of type {bool|function} to specify if the user is able to add new
     * accounts of this type. If false, the user is able to view the account in the
     * list and edit it, but it will be filtered from the dropdown menu of the add
     * button.
     *
     */

    ext.point('io.ox/settings/accounts/settings/detail').extend({
        index: 300,
        id: 'accountssettings',
        draw: function (data) {
            var that = this;

            function redraw() {

                var allAccounts = api.getAll();

                collection = keychainModel.wrap(allAccounts);

                var $pane = drawPane(),
                    accountsList = new ListView({
                        tagName: 'ul',
                        childView: AccountSelectView,
                        collection: collection
                    });

                $pane.append(accountsList.render().$el);

                if (coreSettings.isConfigurable('security/acceptUntrustedCertificates')) {
                    $pane.append(drawCertificateValidation());
                }

                if (collection.length > 1) {
                    $pane.append(drawRecoveryButtonHeadline(), drawRecoveryButton());
                }

                that.empty().append($pane);
            }

            redraw();

            function onChange(id, list) {
                if (!list || list.length === 0 || (id !== 'virtual/io.ox/settings/accounts' && id !== 'virtual/settings/io.ox/settings/accounts')) {
                    api.off('refresh.all refresh.list', redraw);
                    data.tree.off('virtual', onChange);
                }
            }
            api.on('refresh.all refresh.list', redraw);
            data.tree.on('virtual', onChange);
        },
        save: function () {
            // TODO
            //console.log('now accounts get saved?');
        }
    });

    return {};

});
